<?xml version="1.0"?>

<!-- ====================================================================== -->
<!-- = body.xsl                                                           = -->
<!-- = afnix documentation xslt with xhtml output : body processing       = -->
<!-- ====================================================================== -->
<!-- = This  program  is  free  software; you  can redistribute it and/or = -->
<!-- = modify it provided that this copyright notice is kept intact.      = -->
<!-- = This program is distributed in the hope that it will be useful but = -->
<!-- = without  any  warranty;  without  even  the  implied  warranty  of = -->
<!-- = merchantability or fitness for  a  particular purpose. In no event = -->
<!-- = shall  the  copyright  holder be liable for any  direct, indirect, = -->
<!-- = incidental  or special  damages arising  in any way out of the use = -->
<!-- = of this software.                                                  = -->
<!-- ====================================================================== -->
<!-- = copyright (c) 2005-2005 - amaury darsch                            = -->
<!-- ====================================================================== -->

<xsl:stylesheet version="1.0" 
 xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
 xmlns="http://www.w3.org/1999/xhtml">

 <!-- body template -->
 <xsl:template match="body">
  <!-- force the body title template -->
  <xsl:call-template name="body.title"/>
  <!-- default body generation -->
  <xsl:if test="$body.xref = 0">
    <xsl:apply-templates/>
  </xsl:if>
  <!-- body with cross references -->
  <xsl:if test="$body.xref">
    <xsl:call-template name="body.xref"/>
  </xsl:if>
  <!-- force the body footer template -->
  <xsl:call-template name="body.footer"/>
 </xsl:template>

 <!-- body title template -->
 <xsl:template name="body.title">
  <xsl:if test="$body.site = 0">
    <p class="title">
      <xsl:value-of select="$info.title"/>
    </p>
    <hr class="title"/>
  </xsl:if>
  <xsl:if test="$body.site">
    <table>
      <colgroup>
        <col width="25%"/>
        <col width="75%"/>
      </colgroup>
      <tr>
        <td class="menu" align="left">
	  <img src="../img/afnix.png" width="105" height="50" alt="AFNIX"/>
        </td>
	<td class="title" align="right">
           <xsl:value-of select="$info.title"/>
	</td>
      </tr>
    </table>
    <div class="head">
      <a href="../index.htm">home</a>   | 
      <a href="../htm/desc.htm">summary</a>  | 
      <a href="../htm/cnts.htm">contents</a> | 
      <a href="../htm/news.htm">news</a>     | 
      <a href="../htm/lipl.htm">license</a>  |
      <a href="../htm/dwld.htm">download</a>
    </div>
  </xsl:if>
 </xsl:template>

 <!-- body footer template -->
 <xsl:template name="body.footer">
  <xsl:if test="$body.site = 0">
    <hr/>
    <table>
      <tr>
        <td class="footer" align="right">
            Copyright &#169; 2005
            <a href="http://www.darsch.org">Amaury Darsch</a>
        </td>
      </tr>
    </table>
  </xsl:if>
  <xsl:if test="$body.site">
    <hr/>
    <table>
      <tr>
        <td align="left">
          <a  href="../index.htm"><img src="../img/hlp.png"
              width="16" height="16" alt="AFNIX information"/></a>

          <a  href="mailto:support@afnix.org"><img src="../img/mai.png"
	      width="16" height="16" alt="AFNIX support"/></a>
        </td>
        <td class="footer" align="right">
            Copyright &#169; 2005
            <a href="http://www.darsch.org">Amaury Darsch</a>
        </td>
      </tr>
    </table>
  </xsl:if>
 </xsl:template>


 <!-- front matter template -->
 <xsl:template match="front">
  <xsl:apply-templates/>
 </xsl:template>

 <!-- main matter template -->
 <xsl:template match="main">
  <!-- nop -->
  <xsl:apply-templates/>
 </xsl:template>

 <!-- back matter template -->
 <xsl:template match="back">
  <xsl:apply-templates/>
 </xsl:template>

 <!-- synopsis template -->
 <xsl:template match="synopsis">
  <p/>
  <xsl:apply-templates/>
 </xsl:template>

 <!-- chapter template -->
 <xsl:template match="chapter">
  <p/>
  <xsl:apply-templates/>
 </xsl:template>

 <!-- appendix template -->
 <xsl:template match="appendix">
  <p/>
  <xsl:apply-templates/>
 </xsl:template>

 <!-- section template -->
 <xsl:template match="section">
  <p/>
  <xsl:apply-templates/>
 </xsl:template>

 <!-- subsect element -->
 <xsl:template match="subsect">
  <p/>
  <xsl:apply-templates/>
 </xsl:template>

</xsl:stylesheet>

