// ---------------------------------------------------------------------------
// - cgen.cxx                                                                -
// - standard system library - C generator functions implementation          -
// ---------------------------------------------------------------------------
// - This program is free software;  you can redistribute it  and/or  modify -
// - it provided that this copyright notice is kept intact.                  -
// -                                                                         -
// - This program  is  distributed in  the hope  that it will be useful, but -
// - without  any  warranty;  without  even   the   implied    warranty   of -
// - merchantability or fitness for a particular purpose.  In no event shall -
// - the copyright holder be liable for any  direct, indirect, incidental or -
// - special damages arising in any way out of the use of this software.     -
// ---------------------------------------------------------------------------
// - copyright (c) 1999-2005 amaury darsch                                   -
// ---------------------------------------------------------------------------

#include "cthr.hpp"
#include "csys.hpp"
#include "cgen.hpp"
#include "cgen.hxx"

namespace afnix {

  // static mutex creation function
  static void* mtx_create (void);
  // mutex or network services
  static void* mtx = mtx_create ();

  // this function destroy the mutex at exit
  static void mtx_destroy (void) {
    c_mtxdestroy (mtx);
  }

  // this function initialize a mutex statically and register its
  // destruction to be done at exit
  static void* mtx_create (void) {
    void* mtx = c_mtxcreate ();
    c_atexit (mtx_destroy);
    return mtx;
  }

  // return a random number - this function needs a mutex since
  // they are not multi-thread safe

  long c_random (void) {
    c_mtxlock (mtx);
    long result = random ();
    c_mtxunlock (mtx);
    return result;
  }

  // return a random character

  t_byte c_cargen (void) {
    return 1 + (t_byte) (256.0 * random () / (RAND_MAX + 1.0));
  }
}
