// ---------------------------------------------------------------------------
// - Libwww.cpp                                                              -
// - afnix:www library - declaration & implementation                        -
// ---------------------------------------------------------------------------
// - This program is free software;  you can redistribute it  and/or  modify -
// - it provided that this copyright notice is kept intact.                  -
// -                                                                         -
// - This program  is  distributed in  the hope  that it will be useful, but -
// - without  any  warranty;  without  even   the   implied    warranty   of -
// - merchantability or fitness for a particular purpose.  In no event shall -
// - the copyright holder be liable for any  direct, indirect, incidental or -
// - special damages arising in any way out of the use of this software.     -
// ---------------------------------------------------------------------------
// - copyright (c) 1999-2005 amaury darsch                                   -
// ---------------------------------------------------------------------------

#include "Url.hpp"
#include "Meta.hpp"
#include "Libwww.hpp"
#include "Predwww.hpp"
#include "HtmlRef.hpp"
#include "HtmlDiv.hpp"
#include "CgiQuery.hpp"
#include "Function.hpp"
#include "HtmlTable.hpp"
#include "XHtmlPage.hpp"

namespace afnix {
  Object* init_afnix_www (Interp* interp, Vector* argv) {
    // make sure we are not called from something crazy
    if (interp == nilp) return nilp;
    
    // create the afnix:sys nameset
    Nameset* aset = interp->mknset ("afnix");
    Nameset* nset = aset->mknset   ("www");
    
    // bind all classes in the afnix:net nameset
    nset->symcst ("Url",            new Meta (Url::mknew));
    nset->symcst ("Cookie",         new Meta (Cookie::mknew));
    nset->symcst ("CgiQuery",       new Meta (CgiQuery::mknew));
    nset->symcst ("HtmlTag",        new Meta (HtmlTag::mknew));
    nset->symcst ("HtmlTd",         new Meta (HtmlTd::mknew));
    nset->symcst ("HtmlTr",         new Meta (HtmlTr::mknew));
    nset->symcst ("HtmlDiv",        new Meta (HtmlDiv::mknew));
    nset->symcst ("HtmlRef",        new Meta (HtmlRef::mknew));
    nset->symcst ("HtmlPage",       new Meta (HtmlPage::mknew));
    nset->symcst ("HtmlTable",      new Meta (HtmlTable::mknew));
    nset->symcst ("XHtmlPage",      new Meta (XHtmlPage::mknew));
    nset->symcst ("HtmlElement",    new Meta (HtmlElement::mknew));
    
    // bind all predicates in the afnix:www nameset
    nset->symcst ("url-p",          new Function (www_urlp));
    nset->symcst ("cookie-p",       new Function (www_cokp));
    nset->symcst ("html-tag-p",     new Function (www_tagp));
    nset->symcst ("html-td-p",      new Function (www_tbdp));
    nset->symcst ("html-tr-p",      new Function (www_tbrp));
    nset->symcst ("html-div-p",     new Function (www_divp));
    nset->symcst ("html-ref-p",     new Function (www_refp));
    nset->symcst ("html-page-p",    new Function (www_htmp));
    nset->symcst ("cgi-query-p",    new Function (www_cgip));
    nset->symcst ("xhtml-page-p",   new Function (www_xhtp));
    nset->symcst ("html-table-p",   new Function (www_tblp));
    nset->symcst ("html-element-p", new Function (www_elmp));

    // not used but needed
    return nilp;
  }
}

extern "C" {
  afnix::Object* dli_afnix_www (afnix::Interp* interp, afnix::Vector* argv) {
    return init_afnix_www (interp, argv);
  }
}
